package TilitoliGame;

import java.util.Random;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.midlet.MIDlet;

public class GameTable extends Canvas implements Runnable, CommandListener {
    MIDlet parent;
    public GameField[][] fields;
    private int rows;
    private int columns;
    private int rowOfBlack;
    private int columnOfBlack;
    private int amountOfMoves;
    private Image blackImage;
    private boolean lockKeys = false;
    
    private Command EXIT;
    private Command SHOW;
    private Command NEW;
    private Displayable gameDisplay;

    public GameTable(MIDlet p, int numberOfRows, int numberOfColumns, int picSizeX, int picSizeY, String UrlOfBlack, String[] pictureURLs, int moves) throws InvalidPictureURLException {
        if (pictureURLs == null || pictureURLs.length != numberOfColumns * numberOfRows - 1) {
            throw new InvalidPictureURLException("Illegal argument pictureURLs");
        }
        if (moves <= 0) {
            throw new InvalidPictureURLException("Illegal argument: moves must be greater that 0");
        }

        parent = p;

        rows = numberOfRows;
        columns = numberOfColumns;
        
        amountOfMoves = moves;

        fields = new GameField[numberOfRows][numberOfColumns];

        String tempURL;
        for (int i = 0; i < numberOfRows; i++) {
            for (int j = 0; j < numberOfColumns; j++) {
                tempURL = (i == numberOfRows - 1 && j == numberOfColumns - 1) ? UrlOfBlack : pictureURLs[j * columns + i];
                try {
                    fields[i][j] = new GameField(tempURL, i * picSizeX, j * picSizeY);
                } catch (InvalidPictureURLException ex) {
                    throw new InvalidPictureURLException(ex.getMessage() + " " + tempURL);
                }
            }
        }

        rowOfBlack = numberOfRows - 1;
        columnOfBlack = numberOfColumns - 1;
        blackImage = fields[numberOfRows - 1][numberOfColumns - 1].getImage();
    }

    //megcseréljük a feketét és a választottat
    public void moveBlack(int x, int y, boolean checkEnd) {

        if (isValidMovement(x, y)) {
            GameField toRefresh = fields[x][y];
            GameField originalBlack = fields[rowOfBlack][columnOfBlack];

            originalBlack.setImage(toRefresh.getImage());
            toRefresh.setImage(blackImage);

            rowOfBlack = x;
            columnOfBlack = y;

            if (checkEnd && isEndOfGame()) {

                refreshCanvas();

                for (int i = 0; i < 4; i++) {
                    Display.getDisplay(this.parent).vibrate(100);
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException ex) {
                    }
                }

                gameDisplay.removeCommand(SHOW);
                gameDisplay.addCommand(NEW);
                lockKeys = true;
            }
        }
    }

    public boolean isValidMovement(int x, int y) { //pályán marad-e

        return (y < rows && y >= 0 && x < columns && x >= 0);
    }

    public boolean isEndOfGame() {
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                if (!fields[i][j].isAtRightPlace()) {
                    return false;
                }
            }
        }

        return true;
    }

    public void mixFields() {
        int shiftX = 0;
        int shiftY = 0;
        Random randomNumbers = new Random();

        for (int i = 0; i < amountOfMoves; i++) {
            switch (randomNumbers.nextInt() % 4) {
                case 0: { //balra
                    shiftX = +1;
                    break;
                }
                case 1: { //jobbra
                    shiftX = -1;
                    break;
                }
                case 2: { //fel
                    shiftY = +1;
                    break;
                }
                case 3: { //le
                    shiftY = -1;
                    break;
                }
            }

            moveBlack(rowOfBlack + shiftX, columnOfBlack + shiftY, false);
        }
    }
    
    //gondoskodik a Commandokról
    private void initialize(Displayable display) {
        EXIT = new Command("Kilépés", Command.EXIT, 0);
        display.addCommand(EXIT);

        SHOW = new Command("Feladom", Command.OK, 0);
        display.addCommand(SHOW);
        
        NEW = new Command("Új játék", Command.OK, 0);

        display.setCommandListener(this);
        
        gameDisplay = display;
    }

    //megmutatja a helyes megfejtést
    private void show() {
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                fields[i][j].setOriginal();
            }
        }
        
        rowOfBlack = rows - 1;
        columnOfBlack = columns - 1;
        
        refreshCanvas();
    }

    // Runnable interfész absztrakt metódusa
    public void run() {
        System.gc();
        
        initialize(this);

        mixFields();

        do {
            refreshCanvas();
        } while (true);
    }

    //CommandListener interfész absztrakt metódusa
    public void commandAction(Command comm, Displayable display) {
        if (comm == EXIT) {
            for (int i = 0; i <rows; i++) {
                for (int j = 0 ; j<columns; j++) {
                    fields[i][j] = null;
                }
            }
            blackImage = null;
            
            System.gc();
            parent.notifyDestroyed();
        }
        if (comm == SHOW) {
            lockKeys = true;
            display.removeCommand(SHOW);
            display.addCommand(NEW);
            show();
        }
        if (comm == NEW) {
            display.removeCommand(NEW);
            display.addCommand(SHOW);
            mixFields();
            lockKeys = false;
        }
    }

    // Canvas osztály absztrakt metódusa
    protected void keyPressed(int keyCode) {
        if (!lockKeys) {
            switch (keyCode) {
                case KEY_NUM2: {
                    moveBlack(rowOfBlack, columnOfBlack - 1, true);
                    break;
                }
                case KEY_NUM4: {
                    moveBlack(rowOfBlack - 1, columnOfBlack, true);
                    break;
                }
                case KEY_NUM6: {
                    moveBlack(rowOfBlack + 1, columnOfBlack, true);
                    break;
                }
                case KEY_NUM8: {
                    moveBlack(rowOfBlack, columnOfBlack + 1, true);
                    break;
                }
                case UP: {
                    moveBlack(rowOfBlack, columnOfBlack - 1, true);
                    break;
                }
                case LEFT: {
                    moveBlack(rowOfBlack - 1, columnOfBlack, true);
                    break;
                }
                case RIGHT: {
                    moveBlack(rowOfBlack + 1, columnOfBlack, true);
                    break;
                }
                case DOWN: {
                    moveBlack(rowOfBlack, columnOfBlack + 1, true);
                    break;
                }
                default: {
                    break;
                }
            }
        }
    }

    // Canvas osztály absztrakt metódusa
    protected void paint(Graphics g) {
        int width = getWidth();
        int height = getHeight();

        g.setColor(0);
        g.fillRect(0, 0, width, height);

        int startX = (width - columns * fields[0][0].getImage().getWidth()) / 2;
        int startY = (height - rows * fields[0][0].getImage().getHeight()) / 2;

        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                g.drawImage(fields[i][j].getImage(), startX + fields[i][j].posX, startY + fields[i][j].posY, Graphics.TOP | Graphics.LEFT);
            }
        }
    }

    private void refreshCanvas() {
        long startTime = System.currentTimeMillis();

        Thread.yield();

        repaint();
        serviceRepaints();

        long waitTime = System.currentTimeMillis() - startTime;

        try {
            if (waitTime < 60) {
                synchronized (this) {
                    wait(60 - waitTime);
                }
            }
        } catch (InterruptedException ex) {
        }
    }
}
